import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import 'package:flutter_test/flutter_test.dart';
import 'package:coidentity/coidentity.dart';

class Store extends ISharePreferences {
  Map<String, String> store = new Map<String, String>();
  @override
  String getString(String key) {
    return this.store[key];
  }

  @override
  void setString(String key, String value) {
    this.store[key] = value;
  }
}

void main() {
  test('init', () async {
    // 设置使用何种加密方法

    ISharePreferences store = new Store();

    final alice = CoreIdentity();
    final bob = CoreIdentity();
    await alice.setup(EncryptMethod.SM2, store).init();
    await bob.setup(EncryptMethod.SM2, store).init();

    final alice_acc = await alice.account();
    final bob_acc = await bob.account();

    final message =
        await alice.sendTo(bob_acc.publicKey, utf8.encode('我抓到你，你就让我嘿嘿黑！'));

    final utf8raw = await bob.decodeMessage(message);
    print('alice said to bob raw:${utf8.decode(utf8raw)}');

    expect(1, 1);
  });
  test('encrypt decrypt file', () async{
    final me = CoreIdentity();
    ISharePreferences store = new Store();
    me.setup(EncryptMethod.SM2, store).init();
    final secret = Uint8List.fromList([0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,1,2]);
    final iv = Uint8List.fromList([0,1,2,3,4,5,6,7]);
    final encrypted = await me.processFile(Uint8List.fromList([1,2,3,4,5,6,7,8,9,10]), secret, iv);
    print(encrypted);
    final rawList = await me.processFile(encrypted, secret, iv);
    print(rawList);
    expect(rawList, Uint8List.fromList([1,2,3,4,5,6,7,8,9,10]));
  });
  test('Sliced encrypto decrypto file', () async {
    final me = CoreIdentity();
    ISharePreferences store = new Store();
    me.setup(EncryptMethod.SM2, store).init();
    final rows = [
      "轻轻的我走了，",
      "正如我轻轻的来；",
      "我轻轻的招手，",
      "作别西天的云彩。",
      "那河畔的金柳，",
      "是夕阳中的新娘；",
      "波光里的艳影，",
      "在我的心头荡漾。",
      "软泥上的青荇⑴，",
      "油油的在水底招摇⑵；",
      "在康河的柔波里，",
      "我甘心做一条水草！",
      "那榆荫下的一潭，",
      "不是清泉，是天上虹；",
      "揉碎在浮藻间，",
      "沉淀着彩虹似的梦。",
      "寻梦？撑一支长篙⑶，",
      "向青草更青处漫溯⑷；",
      "满载一船星辉，",
      "在星辉斑斓里放歌。",
      "但我不能放歌，",
      "悄悄是别离的笙箫；",
      "夏虫也为我沉默，",
      "沉默是今晚的康桥！",
      "悄悄的我走了，",
      "正如我悄悄的来；",
      "我挥一挥衣袖，",
      "不带走一片云彩。"
    ];

    final fileData = Uint8List.fromList(utf8.encode(rows.join("")));
    final secret = Uint8List.fromList([0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,1,2]);
    final iv = Uint8List.fromList([0,1,2,3,4,5,6,7]);

    Stream<Uint8List> st = me.processFileAsync(fileData, secret, iv, bufferSize: 1024);
    List<List<int>> fileBytesList = <List<int>>[];
    await for (var slice in st) {
        fileBytesList.add(slice.toList());
    }
    var bytes = Uint8List.fromList(fileBytesList.expand((x) => x).toList());
    //print(bytes);

    Stream<Uint8List> st2 = me.processFileAsync(bytes, secret, iv, bufferSize: 1024);
    List<List<int>> fileBytesList2 = <List<int>>[];
    await for (var slice in st2) {
      fileBytesList2.add(slice.toList());
    }
    var rawBytes = Uint8List.fromList(fileBytesList2.expand((x) => x).toList());
    print(utf8.decode(rawBytes.toList()));
    expect(rawBytes, fileData);
  });

  test('Sliced File To Encrypto Decrypto', () async {
    File src = File('/Users/alex/Projects/workspace/coidentity/test/data.txt');
    File tar = File('/Users/alex/Projects/workspace/coidentity/test/data2.txt');
    final secret = Uint8List.fromList([0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,1,2]);
    final iv = Uint8List.fromList([0,1,2,3,4,5,6,7]);
    final me = CoreIdentity();
    ISharePreferences store = new Store();
    me.setup(EncryptMethod.SM2, store).init();
    await me.processFileSegmented(src, tar, secret, iv);

    File decryptedFile = File('/Users/alex/Projects/workspace/coidentity/test/data3.txt');

    Stream<int> progress = me.processFileSegmented(tar, decryptedFile, secret, iv);
    await for (var percent in progress){
      print('$percent % processed');
    }

  });


}
